@@ -498,6 +498,14 @@ TEXTAREA {
 	padding-top: 3px;
 	padding-bottom: 2px;
 }
+.row-cell-transparent {
+	FONT-FAMILY: Arial; 
+	FONT-WEIGHT: normal; 
+	FONT-SIZE: 10pt; 
+	COLOR: #000000; 
+	padding-top: 3px;
+	padding-bottom: 2px;
+}
 .row-button {
 	FONT-FAMILY: Arial; 
 	FONT-WEIGHT: normal; 
@@ -524,11 +532,19 @@ TEXTAREA {
 	background-color: #efefef;
 	padding-top: 3px;
 	padding-bottom: 2px;
-	border-width:1px;
-	border-style:none;
+	
+/*
+	border-top:solid 1px #D6D3CE;
+	border-bottom:solid 1px #D6D3CE;
+*/
+
+	position: relative;
+	top: expression(this.parentElement.parentElement.parentElement.parentElement.scrollTop);
+
 }
 
 
+/*
 .row-cell-hilite {
 	FONT-FAMILY: Arial; 
 	FONT-SIZE: 10pt; 
@@ -537,6 +553,12 @@ TEXTAREA {
 	padding-top: 3px;
 	padding-bottom: 2px;
 }
+*/
+
+.row-cell-hilite {
+	background-color: #dededc;
+}
+
 
 A.row-cell, A.row-cell:link, A.row-cell:active, A.row-cell:hover, A.row-cell:visited {
 	FONT-FAMILY: Arial; 
@@ -559,6 +581,17 @@ checkbox.row-cell,
 
 
 
+td.vert-menu {
+	background-color: #D6D3CE;
+	font-weight: normal; 
+	font-size: 8pt; 
+	color: black; 
+	font-family: MS Sans Serif; 
+	text-decoration: none;
+	padding-top:4px;
+	padding-bottom:4px;
+}
+
 td.main-menu {
 	background-color: #D6D3CE;
 	padding-top:1px;
@@ -672,7 +705,9 @@ A.form-passive-inputs, A.form-passive-inputs:link, A.form-passive-inputs:hover,
 	FONT-SIZE: 10pt; 
 	FONT-FAMILY: Arial; 
 	COLOR: #293869; 
-	TEXT-DECORATION: none
+	TEXT-DECORATION: none;
+	padding-left: 0px;
+	padding-right: 0px;
 }
 
 
@@ -700,8 +735,9 @@ A.form-active-inputs, A.form-active-inputs:link, A.form-active-inputs:hover, A.f
 	FONT-SIZE: 10pt; 
 	FONT-FAMILY: Arial; 
 	COLOR: #293869; 
-	TEXT-DECORATION: none
+	TEXT-DECORATION: none;
 	padding-left: 0px;
+	padding-right: 0px;
 }
 
 .form-active-ellipsis {
@@ -718,3 +754,34 @@ input.form-active-inputs, select.form-active-inputs, textarea.form-active-inputs
 	padding-left: 1px;
 }
 
+
+
+
+div.table-container {
+
+      	PADDING-RIGHT: 0px;
+      	OVERFLOW-Y: auto;
+      	PADDING-LEFT: 0px;
+      	OVERFLOW-X: auto;
+      	PADDING-BOTTOM: 0px;
+      	MARGIN: 0px;
+      	WIDTH: expression(document.body.offsetWidth - (window.name == 'application_frame' ? 12 : 17));
+      	PADDING-TOP: 0px;      	
+      	
+	scrollbar-base-color:#d6d3ce;
+	scrollbar-arrow-color:#485f70;
+	scrollbar-3dlight-color: #efefef;
+	scrollbar-darkshadow-color:#b0b0b0;
+
+}
+
+div.checkboxes {
+
+      	OVERFLOW: auto;
+      	
+	scrollbar-base-color:#d6d3ce;
+	scrollbar-arrow-color:#485f70;
+	scrollbar-3dlight-color: #efefef;
+	scrollbar-darkshadow-color:#b0b0b0;
+
+}
